
import requests
from urllib.parse import urlparse
from unittest.mock import MagicMock, patch
from odoo import release
from odoo.addons.saas_trial.tests.common import TestTrialCommon, tagged
from odoo.addons.saas_worker import metabase


@tagged('post_install', '-at_install', 'post_saas_trial_creation')
class TestTrialRR(TestTrialCommon):

    def test_rr_upgrade_production(self):
        # Test rolling release widget to launch production upgrade (after successfull rr preflight)
        db = metabase.get(self.env.cr.dbname)
        # force prod-like port for runbot as `get_rr_status()` check for that
        metabase.update(db, port=metabase.get_port(release.serie))
        # activate the database
        metabase.set_expiry_in(self.env.cr.dbname, '30 days', set_status='activated')
        # add successfull prefilght upgrade
        curr_major_version = int(str(release.version_info[0]).replace("saas~", ""))
        next_lts_version = "{0}.0".format(curr_major_version + 1)
        rr_preflight_upgrade = metabase.add_migration(
            db, actuator="meta", mode='rr-preflight',
            source_version=release.serie, target_version=next_lts_version,
            state='rolling_release_available', disabled_view_count=0, rr_estimated_time=12 * 60,
        )
        rr_status = metabase.get_rr_status(self.env.cr.dbname)
        self.assertEqual(rr_status['state'], 'rolling_release_available')
        self.assertEqual(rr_status['leave_lts'], False)

        _request_post = requests.post
        _request_mock_staffaccess = MagicMock()
        _request_mock_staffaccess.return_value.json.return_value = {'result': True}

        def _request_post_mock(url, *args, **kwargs):
            urlp = urlparse(url)
            if urlp.path == "/odoo-enterprise/staff-access":
                return _request_mock_staffaccess()  # (url, *args, **kwargs)
            return _request_post(url, *args, **kwargs)

        with patch('requests.post', wraps=_request_post_mock):
            self.start_tour('/web', 'trial_rr_upgrade_available_user', login=self.user_employee.login)
        _request_mock_staffaccess.assert_not_called()

        with patch('requests.post', wraps=_request_post_mock):
            self.start_tour('/web', 'trial_rr_upgrade_available_admin', login=self.user_admin.login)
        _request_mock_staffaccess.assert_called_once()
        upgrade_requests = list([u for u in metabase.get_migrations(db) if u.id != rr_preflight_upgrade.id])
        self.assertEqual(len(upgrade_requests), 1, "There should be 1 new upgrade request")
        upgrade_request = upgrade_requests[0]
        self.assertEqual(upgrade_request.mode, 'self')
        self.assertEqual(upgrade_request.actuator, 'rr')
        self.assertEqual(upgrade_request.state, 'queued')

    def test_rr_upgrade_dismiss(self):
        # When upgrading from a LTS version to a SaaS version, check that user can dismiss
        # the roling release upgrade to stay on LTS.
        curr_major_version = int(str(release.version_info[0]).replace("saas~", ""))
        curr_minor_version = release.version_info[1]
        if curr_minor_version != 0:
            self.skipTest("skipping test for non-LTS releases")

        db = metabase.get(self.env.cr.dbname)
        # force prod-like port for runbot as `get_rr_status()` check for that
        metabase.update(db, port=metabase.get_port(release.serie))
        # activate the database
        metabase.set_expiry_in(self.env.cr.dbname, '30 days', set_status='activated')
        # add successfull prefilght upgrade
        next_saas_version = "saas~{0}.{1}".format(curr_major_version, curr_minor_version + 1)
        rr_preflight_upgrade = metabase.add_migration(
            db, actuator="meta", mode='rr-preflight',
            source_version=release.serie, target_version=next_saas_version,
            state='rolling_release_available', disabled_view_count=0, rr_estimated_time=12 * 60,
        )
        rr_status = metabase.get_rr_status(self.env.cr.dbname)
        self.assertEqual(rr_status['state'], 'rolling_release_available')
        self.assertEqual(rr_status['leave_lts'], True)

        self.start_tour('/web', 'trial_rr_upgrade_dismiss', login=self.user_admin.login)
        upgrade = [u for u in metabase.get_migrations(db) if u.id == rr_preflight_upgrade.id][0]
        self.assertEqual(upgrade.mode, 'rr-preflight', "The mode should not have changed")
        self.assertEqual(
            upgrade.state, 'rr-dismiss',
            "The preflight upgrade state should be now marked as dismissed"
        )
